@@ -6,4 +6,4 @@ Run a command, print its output only if it fails.
 
 from typing import List
 
-__all__: List[str] = []
+__all__: List[str] = []  # noqa: WPS410 (the only __variable__ we use)
