@@ -1,6 +1,6 @@
 # Why does this file exist, and why not put this in `__main__`?
 #
-# You might be tempted to import things from __main__ later,
+# You might be tempted to import things from `__main__` later,
 # but that will cause problems: the code will get executed twice:
 #
 # - When you run `python -m failprint` python will execute
@@ -17,11 +17,15 @@ import os
 import subprocess  # noqa: S404 (we don't mind the security implication)
 import sys
 import textwrap
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Tuple
 
 from ansimarkup import ansiprint
 from jinja2 import Environment
-from ptyprocess import PtyProcessUnicode
+
+try:
+    from ptyprocess import PtyProcessUnicode
+except ModuleNotFoundError:
+    PtyProcessUnicode = None  # noqa: WPS440 (block variable overlap)
 
 DEFAULT_FORMAT = "pretty"
 
@@ -29,20 +33,27 @@ DEFAULT_FORMAT = "pretty"
 class Format:
     """Class to define a display format."""
 
-    def __init__(self, template, progress_template=None, accept_ansi=True):
-        """Initialization method."""
+    def __init__(self, template: str, progress_template: Optional[str] = None, accept_ansi: bool = True) -> None:
+        """
+        Initialize the object.
+
+        Arguments:
+            template: The main template.
+            progress_template: The template to show progress.
+            accept_ansi: Whether to accept ANSI sequences.
+        """
         self.template = template
         self.progress_template = progress_template
         self.accept_ansi = accept_ansi
 
 
-FORMATS: Dict[str, Format] = {
+formats: Dict[str, Format] = {
     "custom": None,  # type: ignore
     "pretty": Format(
-        "<bold>{% if success %}<green>✓</green>{% else %}<red>✗</red>{% endif %} "
+        "<bold>{% if success %}<green>✓</green>{% elif nofail %}<yellow>✗</yellow>{% else %}<red>✗</red>{% endif %} "
         "{{ title or command }}</bold>"
         "{% if failure %} ({{ code }}){% endif %}"
-        "{% if failure and output %}\n"
+        "{% if failure and output and not quiet %}\n"
         "{{ ('  > ' + command + '\n') if title else '' }}"
         "{{ output|indent(2 * ' ') }}{% endif %}",
         progress_template="> {{ title or command }}",
@@ -60,16 +71,24 @@ FORMATS: Dict[str, Format] = {
 class Output(enum.Enum):
     """An enum to store the different possible output types."""
 
-    STDOUT: str = "stdout"
-    STDERR: str = "stderr"
-    COMBINE: str = "combine"
+    STDOUT: str = "stdout"  # noqa: WPS115
+    STDERR: str = "stderr"  # noqa: WPS115
+    COMBINE: str = "combine"  # noqa: WPS115
 
     def __str__(self):
-        return self.value.lower()
+        return self.value.lower()  # noqa: E1101 (false-positive)
 
 
-def printable_command(cmd):
-    """Rebuild a command line from system arguments."""
+def printable_command(cmd: List[str]) -> str:  # noqa: WPS231 (not that complex)
+    """
+    Rebuild a command line from system arguments.
+
+    Arguments:
+        cmd: The command as a list of strings.
+
+    Returns:
+        A printable and shell-runnable command.
+    """
     parts = []
     for part in cmd:
         if " " in part:
@@ -78,7 +97,7 @@ def printable_command(cmd):
             if has_double_quotes and not has_single_quotes:
                 part = f"'{part}'"
             elif has_single_quotes and has_double_quotes:
-                part = part.replace('"', '\\"')
+                part = part.replace('"', r"\"")
                 part = f'"{part}"'
             else:
                 part = f'"{part}"'
@@ -94,11 +113,29 @@ def run(
     fmt: Optional[str] = None,
     use_pty: bool = True,
     progress: bool = True,
+    nofail: bool = False,
+    quiet: bool = False,
 ) -> int:
-    """Run a command in a subprocess, and print its output if it fails."""
+    """
+    Run a command in a subprocess, and print its output if it fails.
+
+    Arguments:
+        cmd: The command to run.
+        number: The command number.
+        output_type: The type of output.
+        title: The command title.
+        fmt: The output format.
+        use_pty: Whether to run in a PTY.
+        progress: Whether to show progress.
+        nofail: Whether to always succeed.
+        quiet: Whether to not print the command output.
+
+    Returns:
+        The command exit code, or 0 if `nofail` is True.
+    """
     format_name: str = fmt or os.environ.get("FAILPRINT_FORMAT", DEFAULT_FORMAT)  # type: ignore
     format_name = accept_custom_format(format_name)
-    format_obj = FORMATS.get(format_name, FORMATS[DEFAULT_FORMAT])
+    format_obj = formats.get(format_name, formats[DEFAULT_FORMAT])
     command = printable_command(cmd)
 
     env = Environment(autoescape=False)  # noqa: S701 (no HTML: no need to escape)
@@ -106,52 +143,20 @@ def run(
 
     if progress and format_obj.progress_template:
         progress_template = env.from_string(format_obj.progress_template)
-        print(progress_template.render({"title": title, "command": command}), end="\r")
+        ansiprint(progress_template.render({"title": title, "command": command}), end="\r")
 
-    if not format_obj.accept_ansi and use_pty:
+    if use_pty and not (format_obj.accept_ansi and PtyProcessUnicode):
         use_pty = False
 
     if output_type is None:
         output_type = Output.COMBINE
 
     if output_type == Output.COMBINE and use_pty:
-        process = PtyProcessUnicode.spawn(cmd)
-
-        pty_output = []
-
-        while True:
-            try:
-                pty_output.append(process.read())
-            except EOFError:
-                break
-
-        process.close()
-
-        output = "".join(pty_output)
-        code = process.exitstatus
-
+        code, output = run_pty_subprocess(cmd)
     else:
-        stdout_opt = subprocess.PIPE
-
-        if output_type == Output.COMBINE:
-            stderr_opt = subprocess.STDOUT
-        else:
-            stderr_opt = subprocess.PIPE
-
-        process = subprocess.Popen(  # noqa: S603 (we trust the input)
-            cmd, stdin=sys.stdin, stdout=stdout_opt, stderr=stderr_opt
-        )
-        stdout, stderr = process.communicate()
-
-        if output_type == Output.STDERR:
-            output = stderr.decode("utf8")
-        else:
-            output = stdout.decode("utf8")
-
-        code = process.returncode
+        code, output = run_subprocess(cmd, output_type)
 
     template = env.from_string(format_obj.template)
-
     ansiprint(
         template.render(
             {
@@ -162,28 +167,107 @@ def run(
                 "failure": code != 0,
                 "number": number,
                 "output": output,
-            }
-        )
+                "nofail": nofail,
+                "quiet": quiet,
+            },
+        ),
     )
 
-    return code
+    return 0 if nofail else code
+
+
+def run_subprocess(cmd: List[str], output_type: Output) -> Tuple[int, str]:
+    """
+    Run a command in a subprocess.
+
+    Arguments:
+        cmd: The command to run.
+        output_type: The type of output.
+
+    Returns:
+        The exit code and the command output.
+    """
+    stdout_opt = subprocess.PIPE
+
+    if output_type == Output.COMBINE:
+        stderr_opt = subprocess.STDOUT
+    else:
+        stderr_opt = subprocess.PIPE
+
+    process = subprocess.Popen(  # noqa: S603 (we trust the input)
+        cmd,
+        stdin=sys.stdin,
+        stdout=stdout_opt,
+        stderr=stderr_opt,
+    )
+    stdout, stderr = process.communicate()
+
+    if output_type == Output.STDERR:
+        output = stderr.decode("utf8")
+    else:
+        output = stdout.decode("utf8")
+
+    code = process.returncode
+
+    return code, output
+
+
+def run_pty_subprocess(cmd: List[str]) -> Tuple[int, str]:
+    """
+    Run a command in a PTY subprocess.
+
+    Arguments:
+        cmd: The command to run.
+
+    Returns:
+        The exit code and the command output.
+    """
+    process = PtyProcessUnicode.spawn(cmd)
+
+    pty_output = []
+
+    while True:
+        try:
+            pty_output.append(process.read())
+        except EOFError:
+            break
+
+    process.close()
+
+    output = "".join(pty_output)
+    code = process.exitstatus
+
+    return code, output
 
 
 def accept_custom_format(string: str) -> str:
-    """Store the value in `FORMATS` if it starts with custom."""
-    if string.startswith("custom=") and FORMATS["custom"] is None:
-        FORMATS["custom"] = Format(string[7:])
+    """
+    Store the value in `formats` if it starts with custom.
+
+    Arguments:
+        string: A format name.
+
+    Returns:
+        The format name, or `custom` if it started with `custom=`.
+    """
+    if string.startswith("custom=") and formats["custom"] is None:
+        formats["custom"] = Format(string[7:])
         return "custom"
     return string
 
 
 def get_parser() -> argparse.ArgumentParser:
-    """Return the CLI argument parser."""
+    """
+    Return the CLI argument parser.
+
+    Returns:
+        An argparse parser.
+    """
     parser = argparse.ArgumentParser(prog="failprint")
     parser.add_argument(
         "-f",
         "--format",
-        choices=FORMATS.keys(),
+        choices=formats.keys(),
         type=accept_custom_format,
         default=None,
         help="Output format. Pass your own Jinja2 template as a string with '-f custom=TEMPLATE'. "
@@ -205,6 +289,23 @@ def get_parser() -> argparse.ArgumentParser:
         type=Output,
         help="Which output to use. Colors are supported with 'combine' only, unless the command has a 'force color' option.",
     )
+    parser.add_argument(
+        "-q",
+        "--quiet",
+        action="store_true",
+        dest="quiet",
+        default=False,
+        help="Don't print the command output, even if it failed.",
+    )
+    parser.add_argument(
+        "-z",
+        "--zero",
+        "--nofail",
+        action="store_true",
+        dest="nofail",
+        default=False,
+        help="Don't fail. Always return a success (0) exit code.",
+    )
     parser.add_argument("-n", "--number", type=int, default=1, help="Command number. Useful for the 'tap' format.")
     parser.add_argument("-t", "--title", help="Command title. Default is the command itself.")
     parser.add_argument("COMMAND", nargs="+")
@@ -212,8 +313,18 @@ def get_parser() -> argparse.ArgumentParser:
     return parser
 
 
-def main(args: Optional[List[str]] = None):
-    """The main function, which is executed when you type `failprint` or `python -m failprint`."""
+def main(args: Optional[List[str]] = None) -> int:
+    """
+    Run the main program.
+
+    This function is executed when you type `failprint` or `python -m failprint`.
+
+    Arguments:
+        args: Arguments passed from the command line.
+
+    Returns:
+        An exit code.
+    """
     parser = get_parser()
     options = parser.parse_args(args)
     return run(
@@ -223,4 +334,6 @@ def main(args: Optional[List[str]] = None):
         title=options.title,
         fmt=options.format,
         use_pty=options.use_pty,
+        nofail=options.nofail,
+        quiet=options.quiet,
     )
